@@ -360,6 +360,9 @@ class Sketch:
 
         self.blit_scale_factor = (1.0, 1.0)
 
+        # Flag to block mouse inputs while open/save dialogs are active
+        self.dialog_active = False
+
         # HACK (disabled in background) trying to get background to work async
         # Works better without
         self._background_args = None
@@ -379,7 +382,11 @@ class Sketch:
             self.oscclient = None
             self.server_thread = None
             self.osc_enabled = True
-            self.start_osc()
+            try:
+                self.start_osc()
+            except OSError as e:
+                print(e)
+                print("Could not start OSC server!")
         else:
             print('pythonosc not installed')
             self.osc_enabled = False
@@ -443,19 +450,64 @@ class Sketch:
     def has_error(self):
         return self.startup_error or self.runtime_error
 
-    def open_file_dialog(self, exts, title='Open file...'):
-        ''' Opens a dialog to select a file to be opened,
-        the first argument is the extension or the file to be opened,
-        e.g. `'png'` or a list of extensions, e.g. `['png', 'jpg']`
+    def open_file_dialog(self, exts, title='Open file…'):
+        """
+        Opens a dialog to select a file.
+        exts: 'png' or ['png', 'jpg'] (extensions without dots)
+        Returns the selected path (str) or '' if cancelled.
+        """
+        if self.dialog_active:
+            return ''
 
-        The function returns the path of the file if it is selected or an empty string othewise.
-        '''
-        # See https://github.com/xMGZx/xdialog
         import xdialog
-        if np.isscalar(exts):
+
+        # Normalize extensions to a list
+        if isinstance(exts, (str, bytes)):
             exts = [exts]
-        filetypes = [(ext + ' files', "*." + ext) for ext in exts]
-        return xdialog.open_file(title, filetypes=filetypes, multiple=False)
+
+        # Build case-insensitive patterns and a friendly label
+        patterns = []
+        filetypes = []
+        for ext in exts:
+            ext = ext.lstrip('.')
+            # add both lower and upper just in case some backends are case sensitive
+            patterns.append(f"*.{ext.lower()}")
+            patterns.append(f"*.{ext.upper()}")
+            filetypes.append((f"{ext.upper()} files", f"*.{ext.lower()}"))
+
+        # Add a catch-all so users can still navigate if platform filtering is quirky
+        filetypes.append(("All files", "*.*"))
+
+        self.dialog_active = True
+        try:
+            res = xdialog.open_file(title, filetypes=filetypes, multiple=False)
+            return res or ''
+        finally:
+            # Always release the guard, even if dialog throws/cancels
+            self.dialog_active = False
+
+
+    # def open_file_dialog(self, exts, title='Open file...'):
+    #     ''' Opens a dialog to select a file to be opened,
+    #     the first argument is the extension or the file to be opened,
+    #     e.g. `'png'` or a list of extensions, e.g. `['png', 'jpg']`
+
+    #     The function returns the path of the file if it is selected or an empty string othewise.
+    #     '''
+    #     if self.dialog_active:
+    #         return ''
+    #     # See https://github.com/xMGZx/xdialog
+    #     import xdialog
+    #     if np.isscalar(exts):
+    #         exts = [exts]
+    #     filetypes = [(ext + ' files', "*." + ext) for ext in exts]
+
+    #     self.dialog_active = True
+    #     res = xdialog.open_file(title, filetypes=filetypes, multiple=False)
+    #     # self.dialog_active = False
+    #     print('End dialog')
+
+    #     return res
 
     def save_file_dialog(self, exts, title='Open file...', filename='untitled'):
         ''' Opens a dialog to select a file to be saved,
@@ -464,9 +516,14 @@ class Sketch:
 
         The function returns the path of the file if it is selected or an empty string othewise.
         '''
+        if self.dialog_active:
+            return ''
+
         import xdialog
         if np.isscalar(exts):
             exts = [exts]
+        self.dialog_active = True
+
         filetypes = [(filename, "*." + ext) for ext in exts]
         file_path = xdialog.save_file(title, filetypes=filetypes)
         root, ext = os.path.splitext(file_path)
@@ -474,6 +531,7 @@ class Sketch:
         if ext.lower() not in [f".{e.lower()}" for e in exts]:
             # If not, append the first extension from the list
             file_path = f"{file_path}.{exts[0].lstrip('.')}"
+
         return file_path
 
     def open_folder_dialog(self, title='Open folder...'):
@@ -482,7 +540,10 @@ class Sketch:
         The function returns the path of the directory if it is selected or an empty string othewise.
         '''
         import xdialog
-        return xdialog.directory(title)
+        self.dialog_active = True
+        res = xdialog.directory(title)
+        #print('End dialog')
+        return res
 
     def _create_canvas(self, w, h, canvas_size=None, fullscreen=False, screen=None, save_background=True):
         self.is_fullscreen = fullscreen
@@ -721,6 +782,7 @@ class Sketch:
         Arguments:
         - `path` (string), the directory where to save the images
         - `num_frames` (int), the number of frames to save
+        - `reload` (bool), whether to reload the sketch, default: True
         '''
         if '~' in path:
             path = os.path.expanduser(path)
@@ -740,7 +802,10 @@ class Sketch:
 
         Arguments:
         - `path` (string), the directory where to save the video
-        - `num_frames` (int), the number of frames to save
+        - `num_frames` (int), the number of frames to save, default: 0
+        - `framerate` (int), the framerate, default: 30
+        - `gamma` (float), the gamma correction, default: 1.0 (see the [OpenCV docs](https://docs.opencv.org/4.x/d3/dc1/tutorial_basic_linear_transform.html))
+        - `reload` (bool), whether to reload the sketch, default: True
         '''
         path = os.path.abspath(path)
         self.grabbing = path
@@ -887,7 +952,7 @@ class Sketch:
         # Attempt to compile script
         try:
             print("Compiling script", self.path)
-            prog_text = open(self.path).read()
+            prog_text = open(self.path, encoding="utf-8").read()
             if 'imgui.' in prog_text:
                 self.prog_uses_imgui = True
             else:
@@ -910,7 +975,7 @@ class Sketch:
             sys.modules[name] = mod
 
             # Exposes classes before load because these might be used outside of functions
-            var_context['VideoInput'] = canvas.VideoInput
+            # var_context['VideoInput'] = canvas.VideoInput
             var_context['__loaded_py5sketch__'] = True
 
             # And also expose canvas as 'c' since the functions in the canvas are quite common names and
@@ -1096,6 +1161,8 @@ class Sketch:
 
     # internal update
     def frame(self, draw_frame):
+        self.dialog_active = False
+
         if self.first_load:
             # Do stuff on first load
             self.first_load = False
@@ -1131,8 +1198,8 @@ class Sketch:
         if self.saving_to_file:
             self.done_saving = True
             draw_frame = True
-            if self.no_loop:
-                self.canvas.background(self.canvas.last_background)
+            # if self.no_loop:
+            #     self.canvas.background(self.canvas.last_background)
 
         # Optional imGUI init and visualization
         if imgui is not None and self._gui_visible:
@@ -1166,6 +1233,7 @@ class Sketch:
             if not self.runtime_error or self._frame_count==0:
                 try:
                     if 'draw' in self.var_context and draw_frame:
+                        self.canvas.blend_mode('over')
                         self.canvas.identity()
                         # Draw background before drawing if specified
                         if self._background_args is not None:
@@ -1251,8 +1319,9 @@ class Sketch:
                 else:
                     surf = self.canvas.surf
                 ctx = cairo.Context(surf)
-                #ctx.set_source_surface(self.setup_surface)
-                #ctx.paint()
+
+                # ctx.set_source_surface(self.setup_surface)
+                # ctx.paint()
                 ctx.set_source_surface(self.recording_surface)
                 ctx.paint()
                 surf.finish()
@@ -1321,7 +1390,7 @@ class Sketch:
             self.settings['osc']['send_addr'] = '127.0.0.1'
         # if self.client_address == 'localhost':
         #     self.client_address = '127.0.0.1'
-        print(self.settings['osc'])
+        #print(self.settings['osc'])
 
         self.dispatcher = Dispatcher()
         self.dispatcher.set_default_handler(self._handle_osc)
@@ -1496,7 +1565,8 @@ def main(path='', fps=0, inject=True, show_toolbar=False):
     sketch._fps = fps
 
     def canvas_pos(x, y):
-        return np.array([x, sketch.window_height-y-sketch.toolbar_height])
+        #return np.array([x, sketch.window_height-y-sketch.toolbar_height])
+        return np.array([x, y-sketch.toolbar_height])
 
     def check_callback(name):
         if not name in sketch.var_context:
@@ -1622,8 +1692,16 @@ def main(path='', fps=0, inject=True, show_toolbar=False):
                 sketch.var_context['mouse_moved']()
 
     def mouse_button_callback(window, button, action, mods):
+        if sketch.dialog_active:
+            return
+
         sketch.modifiers = mods
         pos = sketch._mouse_pos
+
+        if imgui is not None:
+            # print("Mouse event", button, action, mods)
+            sketch.impl.mouse_button_callback(window, button, action, mods)
+
         if action == glfw.PRESS:
 
             # print('Mouse button pressed')
@@ -1652,6 +1730,9 @@ def main(path='', fps=0, inject=True, show_toolbar=False):
                 sig = signature(sketch.var_context['mouse_released'])
                 sketch.var_context['mouse_released'](*params[:len(sig.parameters)])
 
+            #sketch.impl._update_mod_keys(window)
+            #sketch.impl.io.add_mouse_button_event(button, action != 0)
+
     def window_content_scale_callback(window, xscale, yscale):
         #print("Content scale", xscale, yscale)
         pass
@@ -1676,7 +1757,8 @@ def main(path='', fps=0, inject=True, show_toolbar=False):
         sketch.impl._prev_key_callback = key_callback
         sketch.impl._prev_char_callback = char_callback
         sketch.impl._prev_cursor_pos_callback = cursor_position_callback
-        sketch.impl._prev_mouse_button_callback = mouse_button_callback
+        glfw.set_mouse_button_callback(sketch.window, mouse_button_callback)
+        #sketch.impl._prev_mouse_button_callback = None # mouse_button_callback
     else:
         # otherwise explicitly set glfw cb's
         glfw.set_key_callback(sketch.window, key_callback)
@@ -1734,7 +1816,7 @@ def main(path='', fps=0, inject=True, show_toolbar=False):
             # Updates input and calls draw in the sketch
             # Poll for and process events
             glfw.poll_events()
-            sketch._mouse_pos = np.array(glfw.get_cursor_pos(sketch.window))
+            sketch._mouse_pos = canvas_pos(*glfw.get_cursor_pos(sketch.window))
 
             do_frame = False
             delta_t = time.perf_counter() - prev_t
